#!/usr/bin/perl
###############################################################################
# Programming by HotCGIScripts Team | http://www.hotcgiscripts.com/           #
# All Right Reserved                                                          #
###############################################################################

use strict;
use CGI;
use CGI::Carp qw{fatalsToBrowser};
use Time::Local;

use lib './lib';
use MIME::Base64;
use CGI::Session;
use HTML::Template;
use Image::Size;
use DataStore qw{load save};
use mylib;
use CONFIG;

use vars qw{$ROOT_DIR};

$ROOT_DIR = (getpwuid($<))[7] || $ENV{DOCUMENT_ROOT};

if ($ARGV[0] eq 'sendmail' or $ARGV[0] eq 'searchfile')
{
  if (my $pid = fork)
  {
    print $pid;
    exit;
  }
  my $act = shift @ARGV;
  if ($act eq 'sendmail')
  {
    &sendmail();
  }
  elsif ($act eq 'searchfile')
  {
    &searchfile();
  }
  exit;
}

main();

sub main
{
  my $cgi = new CGI;

  for (@CONFIG::system_params)
  {
    $cgi->param($_, undef);
  }

  my $act = $cgi->param('act');

  check_login($cgi);

  if($act eq 'home')
  {
    &home($cgi);
  }
  elsif($act eq 'logout')
  {
    &logout($cgi);
  }

####################################
# SETTINGS MANAGEMENT
####################################
  elsif ($act eq 'edit_settings_form')
  {
    &edit_settings_form($cgi);
  }
  elsif ($act eq 'edit_settings')
  {
    &edit_settings($cgi);
  }


####################################
# UTILS
####################################

  elsif($act eq 'list_utils')
  {
    &list_utils($cgi);
  }

  elsif($act eq 'list_thumbs')
  {
    &list_thumbs($cgi);
  }

  elsif($act eq 'list_logins')
  {
    &list_logins($cgi);
  }

####################################
# FOLDERS MANAGEMENT
####################################

  elsif($act eq 'view_folder')
  {
    &view_folder($cgi);
  }
  elsif($act eq 'view_image')
  {
    &view_image($cgi);
  }
  elsif($act eq 'tree')
  {
    &tree($cgi);
  }
  elsif($act eq 'view_recycle')
  {
    &view_recycle($cgi);
  }

  elsif($act eq 'download_file')
  {
    &download_file($cgi);
  }

  elsif($act eq 'protect_form')
  {
    &protect_form($cgi);
  }
  elsif($act eq 'protect')
  {
    &protect($cgi);
  }

  elsif($act eq 'upload_file_form')
  {
    &upload_file_form($cgi);
  }
  elsif($act eq 'upload_file')
  {
    &upload_file($cgi);
  }

  elsif($act eq 'email_file_form')
  {
    &email_file_form($cgi);
  }
  elsif($act eq 'email_file')
  {
    &email_file($cgi);
  }
  elsif($act eq 'attr_file_form')
  {
    &attr_file_form($cgi);
  }
  elsif($act eq 'attr_file')
  {
    &attr_file($cgi);
  }
  elsif($act eq 'rename_file_form')
  {
    &rename_file_form($cgi);
  }
  elsif($act eq 'rename_file')
  {
    &rename_file($cgi);
  }
  elsif($act eq 'copy_file_form')
  {
    &copy_file_form($cgi);
  }
  elsif($act eq 'copy_file')
  {
    &copy_file($cgi);
  }
  elsif($act eq 'archive_file_form')
  {
    &archive_file_form($cgi);
  }
  elsif($act eq 'archive_file')
  {
    &archive_file($cgi);
  }
  elsif($act eq 'unarchive_file_form')
  {
    &unarchive_file_form($cgi);
  }
  elsif($act eq 'unarchive_file')
  {
    &unarchive_file($cgi);
  }

  elsif($act eq 'create_file_form')
  {
    &create_file_form($cgi);
  }
  elsif($act eq 'create_file')
  {
    &create_file($cgi);
  }
  elsif($act eq 'edit_file_form')
  {
    &edit_file_form($cgi);
  }
  elsif($act eq 'edit_file')
  {
    &edit_file($cgi);
  }


  elsif($act eq 'delete_file')
  {
    &delete_file($cgi);
  }
  elsif($act eq 'erase_file')
  {
    &erase_file($cgi);
  }
  elsif($act eq 'restore_file')
  {
    &restore_file($cgi);
  }

  elsif($act eq 'search_file_form')
  {
    &search_file_form($cgi);
  }
  elsif($act eq 'search_file')
  {
    &search_file($cgi);
  }


  elsif($act eq 'create_folder_form')
  {
    &create_folder_form($cgi);
  }
  elsif($act eq 'create_folder')
  {
    &create_folder($cgi);
  }

  elsif($act eq 'unix_shell')
  {
    &unix_shell($cgi);
  }

  else
  {
    &home($cgi);
  }
}

#####################################################################################################################
# HOME
#####################################################################################################################
sub home
{
  my ($cgi) = @_;

  view_folder($cgi);
}

#####################################################################################################################
# EDIT SETTINGS FORM
#####################################################################################################################
sub edit_settings_form
{
  my ($cgi, @error) = @_;

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_edit_settings_form.tmpl", die_on_bad_params => 0, associate => [$cgi]);
  $templ->param(
                 admin_login => $cgi->param('admin_login') || $CONFIG::admin_login,
                 admin_password => $cgi->param('admin_password') || $CONFIG::admin_password,
		 admin_cpassword => $cgi->param('admin_cpassword') || $CONFIG::admin_password,
                 admin_mail => $cgi->param('admin_mail') || $CONFIG::admin_mail,
                 mail_master => $cgi->param('mail_master') || $CONFIG::mail_master,
                 install_dir => $cgi->param('install_dir') || $CONFIG::base_dir,
                 images_url => $cgi->param('images_url') || $CONFIG::images_dir,
		 sendmail => $cgi->param('sendmail') || $CONFIG::sendmail,
		 tar => $cgi->param('tar') || $CONFIG::tar,
                 ERROR => \@error,
	       );

  my $template = mylib::adm_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}

#####################################################################################################################
# EDIT SETTINGS
#####################################################################################################################
sub edit_settings
{
  my ($cgi) = @_;
  my @error = ();

  if ($cgi->param('admin_login') eq '') { push @error, {ERROR => "Please enter Administrator Login"}};
  if ($cgi->param('admin_password') eq '') { push @error, {ERROR => "Please enter Administrator Password"}}
  elsif ($cgi->param('admin_password') ne $cgi->param('admin_cpassword')) { push @error, {ERROR => "Confirm Administrator Password Correctly"}};

  my $install_dir = $cgi->param('install_dir');
  $install_dir =~ s/\/$//;
  if (! -d $install_dir) { push @error, {ERROR => "Specified Installetinon folder not exists."}}
  else
  {
    if (! -e "$install_dir/admin.cgi") { push @error, {ERROR => "Could not find $install_dir/admin.cgi"}};
    if (! -e "$install_dir/lib") { push @error, {ERROR => "Could not find $install_dir/lib"}};
    if (! -e "$install_dir/templates") { push @error, {ERROR => "Could not find $install_dir/templates"}};
    if (! -e "$install_dir/tmp") { push @error, {ERROR => "Could not find $install_dir/tmp"}};
    if (! -e "$install_dir/RECYCLE") { push @error, {ERROR => "Could not find $install_dir/RECYCLE"}};
  }
  my $images_url = $cgi->param('images_url');
  my ($images_dir) = (mylib::parse_url($images_url))[5];

  if (! -d "$ENV{DOCUMENT_ROOT}$images_dir") { push @error, {ERROR => "Specified Images URL $images_url relating $ENV{DOCUMENT_ROOT}$images_dir not exists."}}
  my $tar = $cgi->param('tar');
  my $result = `$tar 2>&1`;

  if (!$result || $result =~ /Command not found/si)
  {
    push @error, {ERROR => "TAR is unavailable by path specified."}
  }

  my $sendmail = $cgi->param('sendmail');

  if (@error)
  {
    edit_settings_form($cgi, @error);
    return;
  }

  open FILE, "<$CONFIG::base_dir/lib/CONFIG.pm";
  flock FILE, 2;
  my $txt = join '', <FILE>;
  close FILE;

  my $admin_login = $cgi->param('admin_login');
  $admin_login =~ s/([^0-9a-z])/\\$1/gsi;
  $txt =~ s/\$admin_login\s*=\s*".*?";/\$admin_login = "$admin_login";/s;

  my $admin_password = $cgi->param('admin_password');
  $admin_password =~ s/([^0-9a-z])/\\$1/gsi;
  $txt =~ s/\$admin_password\s*=\s*".*?";/\$admin_password = "$admin_password";/s;

  my $admin_mail = $cgi->param('admin_mail');
  $admin_mail =~ s/([^0-9a-z])/\\$1/gsi;
  $txt =~ s/\$admin_mail\s*=\s*".*?";/\$admin_mail = "$admin_mail";/s;

  my $mail_master = $cgi->param('mail_master');
  $mail_master =~ s/([^0-9a-z])/\\$1/gsi;
  $txt =~ s/\$mail_master\s*=\s*".*?";/\$mail_master = "$mail_master";/s;

  $install_dir =~ s/([^0-9a-z])/\\$1/gsi;
  $txt =~ s/\$base_dir\s*=\s*".*?";/\$base_dir = "$install_dir";/s;

  $images_url =~ s/([^0-9a-z])/\\$1/gsi;
  $txt =~ s/\$images_dir\s*=\s*".*?";/\$images_dir = "$images_url";/s;

  $tar =~ s/([^0-9a-z])/\\$1/gsi;
  $txt =~ s/\$tar\s*=\s*".*?";/\$tar = "$tar";/s;

  $sendmail =~ s/([^0-9a-z])/\\$1/gsi;
  $txt =~ s/\$sendmail\s*=\s*".*?";/\$sendmail = "$sendmail";/s;

  my $crypt_method = 1;
  if ($ENV{SERVER_SOFTWARE} !~ /apache/i)
  {
    $crypt_method = 2;
  }
  $txt =~ s/\$crypt_method\s*=\s*".*?";/\$crypt_method = "$crypt_method";/s;


  open FILE, ">$CONFIG::base_dir/lib/CONFIG.pm";
  flock FILE, 1;
  print FILE $txt;
  close FILE;

  my $cookie = $cgi->cookie(-name => 'admin_password', -value => crypt($admin_password, $admin_login));
  print "Set-Cookie: $cookie\n";
  my $cookie = $cgi->cookie(-name => 'admin_login', -value => $admin_login);
  print "Set-Cookie: $cookie\n";

  print "Location: http://$ENV{HTTP_HOST}$ENV{SCRIPT_NAME}?act=edit_settings_form\n\n";
}


#####################################################################################################################
# LIST UTILS
#####################################################################################################################
sub list_utils
{
  my ($cgi, @error) = @_;

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_list_utils.tmpl", die_on_bad_params => 0, associate => [$cgi]);
  my $template = mylib::adm_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}


#####################################################################################################################
# LIST THUMBS
#####################################################################################################################
sub list_thumbs
{
  my ($cgi) = @_;
  my @error = ();

  my $images_dir = $ENV{DOCUMENT_ROOT} . (mylib::parse_url($CONFIG::images_dir))[5];

  my %thumbs = load("$CONFIG::data_dir/thumbs.dat");

  for my $ext ($cgi->param('del_ext'))
  {
    delete $thumbs{$ext};
    unlink <$images_dir/thumbs/$ext.*>;
  }

  if ($cgi->param('image'))
  {
    my $ext = $cgi->param('ext');
    $ext = (split /\./, $ext)[-1]; 
    if (!$ext)
    {
      push @error, { ERROR => "Please, enter file extention." };
    }
    else
    {
      my $file = $cgi->param('image');
      my $img_ext = (split /\./, $file)[-1];
      open FILE, ">$images_dir/thumbs/$ext.$img_ext";
      flock FILE, 2;
      binmode FILE;
      print FILE <$file>;
      close FILE;

      $thumbs{$ext} = "$ext.$img_ext";
    }
  }

  save("$CONFIG::data_dir/thumbs.dat", \%thumbs);

  my @thumbs = ();
  my $color = 0;
  for my $ext (sort keys %thumbs)
  {
    push @thumbs, {
                    EXTENTION => $ext,
		    IMAGE => $thumbs{$ext},
		    image_dir => $CONFIG::images_dir,
		    COLOR => (++$color % 2),
                  };
  }

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_list_thumbs.tmpl", die_on_bad_params => 0, associate => [$cgi]);
  $templ->param(
                 THUMBS => \@thumbs,
                 ERROR  => \@error,
               );
  my $template = mylib::adm_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}

#####################################################################################################################
# LIST LOGINS
#####################################################################################################################
sub list_logins
{
  my ($cgi) = @_;
  my @error = ();

  my $cur_period = $cgi->param('period');
  my $start = my $finish = 0;

  if ($cgi->param('period') eq 'Today')
  {
    $start = timelocal(0, 0, 0, (localtime())[3], (localtime())[4], (localtime())[5]+1900);
    $finish = timelocal(59, 59, 23, (localtime())[3], (localtime())[4], (localtime())[5]+1900);
  }
  elsif ($cgi->param('period') eq 'Yesterday')
  {
    $start = timelocal(0, 0, 0, (localtime(time() - 86400))[3], (localtime(time() - 86400))[4], (localtime(time() - 86400))[5]+1900);
    $finish = timelocal(59, 59, 23, (localtime(time() - 86400))[3], (localtime(time() - 86400))[4], (localtime(time() - 86400))[5]+1900);
  }
  elsif ($cgi->param('period') eq 'Last week')
  {
    my $wday = (localtime())[6];
    $wday = 6 if ((--$wday) < 0);
    my $wlday = $wday;
    $wlday = 7 if ((++$wlday) > 7);

    $start = timelocal(0, 0, 0, (localtime(time() - 86400 * ($wday + 7)))[3], (localtime(time() - 86400 * ($wday + 7)))[4], (localtime(time() - 86400 * ($wday + 7)))[5]+1900);
    $finish = timelocal(59, 59, 24, (localtime(time() - 86400 * $wlday))[3], (localtime(time() - 86400 * $wlday))[4], (localtime(time() - 86400 * $wlday))[5]+1900);
  }
  elsif ($cgi->param('period') eq 'Month')
  {
    my ($day, $month, $year) = (localtime())[3..5];

    $start = timelocal(0, 0, 0, 1, $month, $year + 1900);
    $finish = timelocal(59, 59, 23, (localtime())[3], (localtime())[4], (localtime())[5]+1900);
  }
  elsif ($cgi->param('period') eq 'This Week' || !defined $cgi->param('start_day')) 
  {
    $cur_period = 'This Week';
    my $wday = (localtime())[6];
    $wday = 6 if ((--$wday) < 0);

    $start = timelocal(0, 0, 0, (localtime(time() - 86400 * $wday))[3], (localtime(time() - 86400 * $wday))[4], (localtime(time() - 86400 * $wday))[5]+1900);
    $finish = timelocal(59, 59, 23, (localtime())[3], (localtime())[4], (localtime())[5]+1900);
  }
  else
  {
    $start = timelocal(0, 0, 0, $cgi->param('start_day'), $cgi->param('start_month') - 1, $cgi->param('start_year'));
    $finish = timelocal(59, 59, 23, $cgi->param('finish_day'), $cgi->param('finish_month') - 1, $cgi->param('finish_year'));
  }

  my $color = 0;
  my @log = ();
  open FILE, "<$CONFIG::actions_log";
  flock FILE, 2;
  while (<FILE>)
  {
    chomp;
    next if /^\s*$/;
    my ($date, $ip, $message) = split /\t/, $_, 3;
    next if ($date < $start || $date > $finish);

    push @log, {
                    DATE => mylib::time_FromUnixToUSA($date),
                    TIME => $date,
		    IP => $ip,
		    MESSAGE => $message,
		    COLOR => (++$color % 2),
               };

  }
  close FILE;

  @log = sort { $b->{TIME} <=> $a->{TIME} } @log;

  my ($from_year, $from_month, $from_day) = mylib::get_date_arrays((localtime($start))[5] + 1900, (localtime($start))[4] + 1, (localtime($start))[3], 0, 5);
  my ($to_year,   $to_month,   $to_day)   = mylib::get_date_arrays((localtime($finish))[5] + 1900, (localtime($finish))[4] + 1, (localtime($finish))[3], 0, 5);

  my @periods = ();
  for my $period (('Today', 'Yesterday', 'This Week', 'Last week', 'Month'))
  {
    my $selected = ($period eq $cur_period) ? 'selected' : '';
    push @periods, { NAME => $period, VALUE => $period, SELECTED => $selected };
  }

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_list_logins.tmpl", die_on_bad_params => 0, associate => [$cgi]);
  $templ->param(
                 FROM_DAY => $from_day,
                 FROM_MONTH => $from_month,
                 FROM_YEAR => $from_year,
                 TO_DAY => $to_day,
                 TO_MONTH => $to_month,
                 TO_YEAR => $to_year,
                 PERIODS => \@periods,
                 LOG => \@log,
                 ERROR  => \@error,
               );
  my $template = mylib::adm_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}

####################################################################################################################
# VIEW FOLDER
#####################################################################################################################
sub view_folder
{
  my ($cgi, @error) = @_;

  my $session = new CGI::Session("dr:File;ser:Default;id:MD5", $cgi, {Directory => "$CONFIG::tmp_dir"} );
  $session->save_param($cgi);
  $session->load_param($cgi);
  $session->param('dir', undef);

  my $cookie = $cgi->cookie(CGISESSID => $session->id);
  print "Set-Cookie: $cookie\n";

  my $account_size = mylib::dir_size($ROOT_DIR);
  my $recycle_size = mylib::dir_size($CONFIG::recycle_dir);

  if (! defined $cgi->param('&current_dir'))
  {
    $cgi->param('&current_dir', $ROOT_DIR);
  }

  my $go_dir = $cgi->param('dir');

  if ($go_dir =~ m~^/~)
  {
    $go_dir =~ s~\.\./|\.\.$~~gs;
    $go_dir =~ s~[\\\/]$~~;
    $cgi->param('&current_dir', $go_dir);
  }
  else
  {
    $go_dir =~ s~[\\\/]+$~~;
    $go_dir = (split m~[\\\/]~, $go_dir)[-1];
  }
  $go_dir =~ s~[\\\/]+$~~;

  if ($go_dir eq '..')
  {
    my $current_dir = $cgi->param('&current_dir');
    $current_dir =~ s/\/[^\\\/]+$//;
    $cgi->param('&current_dir', $current_dir);
  }
  elsif (($go_dir ne '') && ($go_dir ne '.') && (-d($cgi->param('&current_dir')."/$go_dir")) )
  {
    $cgi->param('&current_dir', $cgi->param('&current_dir') . "/$go_dir");
  }

  my $current_dir = $cgi->param('&current_dir');

  opendir DIR, "/$current_dir" or print_error($cgi, {ERROR => "Could not read dir '$current_dir': $!"});
  my @file_names = grep {!/^\.$/} readdir DIR;
  closedir DIR;

  my @files = ();

  my %thumbs = load("$CONFIG::data_dir/thumbs.dat");

  for my $file_name (@file_names)
  {
    my $file_ext = (split /\./, $file_name)[-1];
    my ($mode, $bite_size, $date) = (stat("$current_dir/$file_name"))[2, 7, 9];

    my $str_date = mylib::time_FromUnixToUSA($date);
    my $size = mylib::file_size($bite_size);
    my $type = 'BINARYFILE';
    my $ziped = 0;
    my %image = ();

    if ( -d "$current_dir/$file_name") 
    {
      $type = 'FOLDER';
    }
    elsif ( -l "$current_dir/$file_name")
    {
      $type = 'LINK';
    }
    elsif ( -T "$current_dir/$file_name")
    {
      $type = 'TEXTFILE';
    }
    else
    {
      if ((imgsize("$current_dir/$file_name"))[0])
      {
        my ($w, $h, $type) = imgsize("$current_dir/$file_name");
        %image = (IMAGE => 1, IMAGE_TYPE => $type, IMG_WIDTH => $w, IMG_HEIGHT => $h);
      }
      else
      {
        $ziped = mylib::test_archive("$current_dir/$file_name");
      }
    }

    my $weight = 0;
    $weight = 1 if ($type eq 'FOLDER');
    $weight = 2 if ($file_name eq '..');

    push @files, {
                  NAME => $file_name,
                  ATTR => substr(sprintf("%03lo", $mode), -3),
                  SIZE => $size,
                  $type => 1,
                  THUMB => "$thumbs{$file_ext}",
                  DATE => $str_date,
                  TIME => $date,
                  BITE_SIZE => $bite_size,
                  WEIGHT => $weight,
                  "WEIGHT$weight" => 1,
                  ARCHIVE => $ziped,
                  image_dir => $CONFIG::images_dir,
                  %image,
                }

  }

  my %orders = (
                 'NAME' => 'cmp',
                 'BITE_SIZE' => '<=>',
                 'TIME' => '<=>',
               );

  my $order = $cgi->param('order');
  my $order_dir = $cgi->param('order_dir');
  $order = 'NAME' if (!$orders{$order});

  @files = sort {
                  unless ($b->{WEIGHT} <=> $a->{WEIGHT})
                  {
                    unless ($order_dir) 
                    {
                      if ($orders{$order} eq 'cmp')
                      {
                        $a->{$order} cmp $b->{$order}
                      }
                      else
                      {
                        $a->{$order} <=> $b->{$order}
                      }
                    }
                    else
                    {
                      if ($orders{$order} eq 'cmp')
                      {
                        $b->{$order} cmp $a->{$order}
                      }
                      else
                      {
                        $b->{$order} <=> $a->{$order}
                      }
                    }
                  }
                } @files;

  my $i = 0;
  map {$_->{COLOR} = ($i++ % 2)} @files;

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_view_folder.tmpl", die_on_bad_params => 0, associate => [$cgi]);

  mylib::prepare_array_view(\@files);
  $templ->param(
                 FILES => \@files,
                 CURRENT_PATH => $current_dir,
                 ERROR => \@error,
                 order_dir => !$cgi->param('order_dir'),
                 "order_$order" => 1,
                 ACCOUNT_SIZE => mylib::file_size($account_size, 1),
                 RECYCLE_SIZE => mylib::file_size($recycle_size, 1),
	       );

  my $template = mylib::adm_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
  $session->param('&current_dir', $current_dir) or print $session->error();
}

#####################################################################################################################
# VIEW IMAGE
#####################################################################################################################
sub view_image
{
  my ($cgi) = @_;

  my $session = new CGI::Session("dr:File;ser:Default;id:MD5", $cgi, {Directory => "$CONFIG::tmp_dir"} );

  my $current_dir = $session->param('&current_dir');

  my @tmp_files = ($cgi->param('file'));
  my @file_names = mylib::get_files($current_dir, \@tmp_files);

  print "Content-type: image\n\n";

  open FILE, "$current_dir/$file_names[0]";
  binmode FILE;
  binmode STDOUT;
  print <FILE>;
  close FILE;
}

#####################################################################################################################
# VIEW RECYCLE
#####################################################################################################################
sub view_recycle
{
  my ($cgi, @error) = @_;

  my $account_size = mylib::dir_size($ROOT_DIR);
  my $recycle_size = mylib::dir_size($CONFIG::recycle_dir);

  my %data = load("$CONFIG::recycle_dir/init.dat");
  for my $file (keys %data)
  {
    delete $data{$file} if (! -e "$CONFIG::recycle_dir/$file");
  }
  save("$CONFIG::recycle_dir/init.dat", \%data);

  my @files = ();

  my %thumbs = load("$CONFIG::data_dir/thumbs.dat");

  for my $file_name (keys %data)
  {
    my $file_ext = (split /\./, $file_name)[-1];
    my ($bite_size) = (stat("$CONFIG::recycle_dir/$file_name"))[7];
    my $date = $data{$file_name}->{DATE};
    my $path = $data{$file_name}->{SOURCE_FILE};
    my $name = (split /[\\\/]/, $path)[-1];
    $path =~ s/[^\\\/]*$//;
    my $root_dir = $ROOT_DIR;
    $path =~ s/^$root_dir//s;

    my $str_date = mylib::time_FromUnixToUSA($date);
    my $size = mylib::file_size($bite_size);
    my $type = 'FILE';
    if ( -d "$CONFIG::recycle_dir/$file_name") 
    {
      $type = 'FOLDER';
    }
    elsif ( -f "$CONFIG::recycle_dir/$file_name")
    {
      $type = 'FILE';
    }

    my $weight = 0;
    $weight = 1 if ($type eq 'FOLDER');


    push @files, {
                  NAME => $name,
                  ACTUAL_NAME => $file_name,
                  SIZE => $size,
                  $type => 1,
                  THUMB => "$thumbs{$file_ext}",
                  DATE => $str_date,
                  TIME => $date,
                  PATH => $path,
                  BITE_SIZE => $bite_size,
                  WEIGHT => $weight,
                  "WEIGHT$weight" => 1,
                  image_dir => $CONFIG::images_dir,
                }

  }

  my %orders = (
                 'NAME' => 'cmp',
                 'BITE_SIZE' => '<=>',
                 'TIME' => '<=>',
               );

  my $order = $cgi->param('order');
  my $order_dir = $cgi->param('order_dir');
  $order = 'NAME' if (!$orders{$order});

  @files = sort {
                  unless ($b->{WEIGHT} <=> $a->{WEIGHT})
                  {
                    unless ($order_dir) 
                    {
                      if ($orders{$order} eq 'cmp')
                      {
                        $a->{$order} cmp $b->{$order}
                      }
                      else
                      {
                        $a->{$order} <=> $b->{$order}
                      }
                    }
                    else
                    {
                      if ($orders{$order} eq 'cmp')
                      {
                        $b->{$order} cmp $a->{$order}
                      }
                      else
                      {
                        $b->{$order} <=> $a->{$order}
                      }
                    }
                  }
                } @files;

  my $i = 0;
  map {$_->{COLOR} = ($i++ % 2)} @files;

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_view_recycle.tmpl", die_on_bad_params => 0, associate => [$cgi]);

  mylib::prepare_array_view(\@files);
  $templ->param(
                 FILES => \@files,
                 ERROR => \@error,
                 order_dir => !$cgi->param('order_dir'),
                 "order_$order" => 1,
                 ACCOUNT_SIZE => mylib::file_size($account_size, 1),
                 RECYCLE_SIZE => mylib::file_size($recycle_size, 1),
	       );

  my $template = mylib::adm_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}

#####################################################################################################################
# ATTR FILE FORM
#####################################################################################################################
sub attr_file_form
{
  my ($cgi) = @_;

  my $session = new CGI::Session("dr:File;ser:Default;id:MD5", $cgi, {Directory => "$CONFIG::tmp_dir"} );

  my $current_dir = $session->param('&current_dir');

  my @tmp_files = ();
  if ($cgi->param('one_file'))
  {
    push @tmp_files, $cgi->param('one_file');
  }
  else
  {
    @tmp_files = $cgi->param('file');
  }
  @tmp_files = grep {$_ ne '..'} @tmp_files;

  my @file_names = mylib::get_files($current_dir, \@tmp_files);

  my @files = ();
  map { push @files, {NAME => $_} } @file_names;

  my %attrs = ();
  if (scalar @files == 1)
  {
    my @attr = split //, substr(sprintf("%03o", (stat("$current_dir/$tmp_files[0]"))[2]), -3);
    my @a = ('o', 'g', 'd');
    my $i;
    for ($i = 0; $i<@attr; $i++) 
    {
      if ($attr[$i] & 4) { $attrs{"$a[$i]r"} = 1; } else { $attrs{"$a[$i]r"} = 0; }
      if ($attr[$i] & 2) { $attrs{"$a[$i]w"} = 1; } else { $attrs{"$a[$i]w"} = 0; }
      if ($attr[$i] & 1) { $attrs{"$a[$i]x"} = 1; } else { $attrs{"$a[$i]x"} = 0; }
    }
  }

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_attr_file_form.tmpl", die_on_bad_params => 0, associate => [$cgi]);
  $templ->param(
                 %attrs,
                 FILES => \@files,
                 MULTIFILES => (1 < scalar @files),
	       );

  my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}

#####################################################################################################################
# ATTR FILE
#####################################################################################################################
sub attr_file
{
  my ($cgi) = @_;

  my $session = new CGI::Session("dr:File;ser:Default;id:MD5", $cgi, {Directory => "$CONFIG::tmp_dir"} );

  my $current_dir = $session->param('&current_dir');

  my @tmp_files = $cgi->param('file');
  @tmp_files = grep {$_ ne '..'} @tmp_files;

  my @file_names = mylib::get_files($current_dir, \@tmp_files);

  my %keys = map { /(.*?)=(.*)/; 
                   my $key = $1; 
                   my $val = $2; 
                   if($val ne '0' && $val ne '1') { $val = '' }; 
                   {$key => $val} 
                 } (split /&/, $cgi->param('attrs'));

  my @results = ();
  for my $file_name (@file_names)
  {
    my %attrs;
    my @attr = split //, substr(sprintf("%03o", (stat("$current_dir/$file_name"))[2]), -3);
    my @a = ('o', 'g', 'd');
    my $i;
    for ($i = 0; $i<@attr; $i++) 
    {
      if ($attr[$i] & 4) { $attrs{"$a[$i]r"} = 1; }
      if ($keys{"$a[$i]r"} ne '') { $attrs{"$a[$i]r"} = $keys{"$a[$i]r"}; }
      if ($attr[$i] & 2) { $attrs{"$a[$i]w"} = 1; }
      if ($keys{"$a[$i]w"} ne '') { $attrs{"$a[$i]w"} = $keys{"$a[$i]w"}; }
      if ($attr[$i] & 1) { $attrs{"$a[$i]x"} = 1; } 
      if ($keys{"$a[$i]x"} ne '') { $attrs{"$a[$i]x"} = $keys{"$a[$i]x"}; }
    }

    my $o = $attrs{or}*4 + $attrs{ow}*2 + $attrs{ox};
    my $g = $attrs{gr}*4 + $attrs{gw}*2 + $attrs{gx};
    my $d = $attrs{dr}*4 + $attrs{dw}*2 + $attrs{dx};

    $! = '';
    my $attr = "0$o$g$d";
    umask(0000);
    chmod(oct $attr, "$current_dir/$file_name");
    push @results, {NAME => $file_name, ATTR => $attr, SUCCESS => ($!) ? 0 : 1, ERROR => $!};
  }

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_attr_file_results.tmpl", die_on_bad_params => 0, associate => [$cgi]);

  $templ->param(
                 RESULTS => \@results,
	       );

  my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}


#####################################################################################################################
# PROTECT FORM
#####################################################################################################################
sub protect_form
{
  my ($cgi, @error) = @_;

  my $session = new CGI::Session("dr:File;ser:Default;id:MD5", $cgi, {Directory => "$CONFIG::tmp_dir"} );

  my $current_dir = $session->param('&current_dir');

  my @web_users = ();
  open FILE, "<$current_dir/.htpasswd";
  flock FILE, 2;
  while (<FILE>)
  {
    chomp;
    my $user = (split /:/, $_)[0];
    push @web_users, {NAME => $user};
  }
  close FILE;


  mylib::prepare_array_view(\@web_users);
  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_protect_form.tmpl", die_on_bad_params => 0, associate => [$cgi]);
  $templ->param(
                 USERS => \@web_users,
                 ERROR => \@error,
	       );

  my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}

#####################################################################################################################
# PROTECT
#####################################################################################################################
sub protect
{
  my ($cgi) = @_;
  my @error = ();

  my $session = new CGI::Session("dr:File;ser:Default;id:MD5", $cgi, {Directory => "$CONFIG::tmp_dir"} );

  my $current_dir = $session->param('&current_dir');

  my @results = ();

  if ($cgi->param('newuser'))
  {
    my $newuser = $cgi->param('newuser');
    if ($newuser =~ m~:~)
    {
      push @error, {ERROR => "Username could not contains \":\" char"};
    }
    elsif (!$cgi->param('newpassword'))
    {
      push @error, {ERROR => "Please, enter New User Password"};
    }

    open FILE, "<$current_dir/.htpasswd";
    flock FILE, 2;
    while (<FILE>)
    {
      chomp;
      my $user = (split /:/, $_)[0];
      if ($user eq $newuser)
      {
        push @error, {ERROR => "Username already exists"};
        last;
      }
    }
    close FILE;

    if (@error)
    {
      protect_form($cgi, @error);
      return;
    }
    my @error = ();
    my $error = mylib::create_htaccess("$current_dir");
    push @error, {ERROR => $error} if $error;
    my $error = mylib::add_htaccess_user("$current_dir", $cgi->param('newuser'), $cgi->param('newpassword'));
    push @error, {ERROR => $error} if $error;

    push @results, {ACTION => 'ADD', USER => "$newuser" , SUCCESS => (@error) ? 0 : 1, ERROR => \@error};
  }

  for ($cgi->param('users'))
  {
    my @error = ();
    my $error = mylib::del_htaccess_user("$current_dir", $_);
    push @error, {ERROR => $error} if $error;
    push @results, {ACTION => 'DELETE', USER => "$_" , SUCCESS => (@error) ? 0 : 1, ERROR => \@error};
  }
  
  if (-z "$current_dir/.htpasswd")
  {
    my @error = ();
    my $error = mylib::remove_htaccess("$current_dir");
    push @error, {ERROR => $error} if $error;
    push @results, {ACTION => 'REMOVE PROTECTION', USER => "", SUCCESS => (@error) ? 0 : 1, ERROR => \@error};
  }

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_protect_results.tmpl", die_on_bad_params => 0, associate => [$cgi]);
  $templ->param(
                 RESULTS => \@results,
	       );

  my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}

#####################################################################################################################
# UPLOAD FILE FORM
#####################################################################################################################
sub upload_file_form
{
  my ($cgi) = @_;

  my $session = new CGI::Session("dr:File;ser:Default;id:MD5", $cgi, {Directory => "$CONFIG::tmp_dir"} );

  my $current_dir = $session->param('&current_dir');

  my %upload_types = %CONFIG::upload_types;

  my $upload_type = int $cgi->param('upload_type');
  $upload_type = 0 if (!defined $upload_types{$upload_type});

  my @upload_types = ();
  for my $type (keys %upload_types)
  {
    push @upload_types, { ID => $type, NAME => $upload_types{$type}, SELECTED => ($type == $upload_type) ? 1 : 0 };
  }

  my @files = ();
  for my $id (1..$CONFIG::max_upload_once)
  {
    my %data = ();
    $data{ID} = $id;
    $data{image_dir} = $CONFIG::images_dir;
    $data{NAME} = $cgi->param("file$id");
    push @files, {%data};
  }

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_upload_$upload_type\_file_form.tmpl", die_on_bad_params => 0, associate => [$cgi]);
  $templ->param(
                 FILES => \@files,
                 UPLOAD_TYPE => \@upload_types,
	       );

  my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}

#####################################################################################################################
# UPLOAD FILE
#####################################################################################################################
sub upload_file
{
  my ($cgi) = @_;

  my $session = new CGI::Session("dr:File;ser:Default;id:MD5", $cgi, {Directory => "$CONFIG::tmp_dir"} );

  my $current_dir = $session->param('&current_dir');

  my %upload_types = %CONFIG::upload_types;
  my $upload_type = int $cgi->param('upload_type');
  if (!defined $upload_types{$upload_type})
  {
    print_error($cgi, {ERROR => 'Unknown upload type.'});
    return;
  }

  my @results = ();

  if ($upload_type == 1)
  {
    for my $id (1..$CONFIG::max_upload_once)
    {
      my $file = $cgi->param("file$id");
      next if $file =~ /^\s*$/;
      next if ($file =~ /^[^:]*:\/\/$/);

      if ($file !~ /^[^:]*:\/\//)
      {
        $file = "http://$file";
      }
      my $file_name = (split /[\\\/]+/, $file)[-1];
      $file_name = 'index.html' if ($file_name =~ /^\s*$/);

      if ((! int($cgi->param('overwrite'))) && (-e "$current_dir/$file_name"))
      {
        my @error = ({FILE => "$file_name", ERROR => "not overwriten"});
        push @results, {OLDNAME => $file, NEWNAME => "$file_name" , SUCCESS => 0 , ERROR => \@error};
      }
      elsif ((-e "$current_dir/$file_name") && (! -f "$current_dir/$file_name"))
      {
        my @error = ({FILE => "$file_name", ERROR => "could not overwrite"});
        push @results, {OLDNAME => $file, NEWNAME => "$file_name" , SUCCESS => 0 , ERROR => \@error};
      }
      else
      {
        my @error = ();
        my ($data, $error) = mylib::get_doc($file);
        if ($error)
        {
          push @error, {FILE => "$file", ERROR => $error};
        }
        else
        {
          open FILE, ">$current_dir/$file_name" or push @error, {FILE => "$file_name", ERROR => "$!"};
          flock FILE, 2;
          binmode FILE;
          print FILE $data;
          close FILE;
        }
        push @results, {OLDNAME => $file_name, NEWNAME => "$file_name" , SUCCESS => (@error) ? 0 : 1, ERROR => \@error};
      }
    }
  }
  elsif ($upload_type == 2)
  {
    for my $id (1..$CONFIG::max_upload_once)
    {
      my $file = $cgi->param("file$id");
      next if $file =~ /^\s*$/;

      my $file_name = (split /[\\\/]+/, $file)[-1];
      $file_name = 'index.html' if ($file_name =~ /^\s*$/);

      if ((! int($cgi->param('overwrite'))) && (-e "$current_dir/$file_name"))
      {
        my @error = ({FILE => $file_name, ERROR => "not overwriten"});
        push @results, {OLDNAME => $file, NEWNAME => "$file_name" , SUCCESS => 0 , ERROR => \@error};
      }
      elsif ((-e "$current_dir/$file_name") && (! -f "$current_dir/$file_name"))
      {
        my @error = ({FILE => $file_name, ERROR => "could not overwrite"});
        push @results, {OLDNAME => $file, NEWNAME => "$file_name" , SUCCESS => 0 , ERROR => \@error};
      }
      else
      {
        my @error = ();
        my ($data, $error) = mylib::get_ftp_doc($cgi->param('host'), $cgi->param('login'), $cgi->param('password'), $file, "$current_dir/$file_name");
        if ($error)
        {
          push @error, {FILE => $file, ERROR => $error};
        }
        push @results, {OLDNAME => $file, NEWNAME => "$file_name" , SUCCESS => (@error) ? 0 : 1, ERROR => \@error};
      }
    }
  }
  else
  {
    for my $id (1..$CONFIG::max_upload_once)
    {
      my $file = $cgi->param("file$id");
      next if $file =~ /^\s*$/;

      my $file_name = (split /[\\\/]+/, $file)[-1];

      if ((! int($cgi->param('overwrite'))) && (-e "$current_dir/$file_name"))
      {
        my @error = ({FILE => "$file_name", ERROR => "not overwriten"});
        push @results, {OLDNAME => $file, NEWNAME => "$file_name" , SUCCESS => 0 , ERROR => \@error};
      }
      elsif ((-e "$current_dir/$file_name") && (! -f "$current_dir/$file_name"))
      {
        my @error = ({FILE => "$current_dir/$file_name", ERROR => "could not overwrite"});
        push @results, {OLDNAME => $file, NEWNAME => "$file_name" , SUCCESS => 0 , ERROR => \@error};
      }
      else
      {
        my @error = ();
        open FILE, ">$current_dir/$file_name" or push @error, {FILE => "$file_name", ERROR => "$!"};
        flock FILE, 2;
        binmode FILE;
        print FILE <$file>;
        close FILE;
        push @results, {OLDNAME => $file, NEWNAME => "$file_name" , SUCCESS => (@error) ? 0 : 1, ERROR => \@error};
      }
    }
  }


  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_upload_file_results.tmpl", die_on_bad_params => 0, associate => [$cgi]);

  $templ->param(
                 RESULTS => \@results,
	       );

  my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}



#####################################################################################################################
# DOWNLOAD FILE
#####################################################################################################################
sub download_file
{
  my ($cgi) = @_;

  my $session = new CGI::Session("dr:File;ser:Default;id:MD5", $cgi, {Directory => "$CONFIG::tmp_dir"} );

  my $current_dir = $session->param('&current_dir');

  my @tmp_files = ($cgi->param('one_file'));
  @tmp_files = grep {$_ ne '..'} @tmp_files;
  my @file_names = mylib::get_files($current_dir, \@tmp_files);
  my ($download_file) = @file_names;

  if (! -f "$current_dir/$download_file")
  {
    print_error($cgi, {ERROR => "No such file $download_file"});
    return;
  }
 
  my $size = (stat("$current_dir/$download_file"))[7];

  open FILE, "<$current_dir/$download_file" or print_error($cgi, {ERROR => "Could not read file $download_file"});
  flock FILE, 2;
  print "Accept-Ranges: bytes\n";
  print "Content-Length: $size\n";
  print "Connection: close\n";
  print "Content-Disposition: attachment; filename=$download_file\n";
  print "Content-type: application/octet-stream\n\n";
  binmode FILE;
  binmode STDOUT;
  while (<FILE>) 
  {
    print;
  }
  close FILE;
}

#####################################################################################################################
# EMAIL FILE FORM
#####################################################################################################################
sub email_file_form
{
  my ($cgi) = @_;

  my $session = new CGI::Session("dr:File;ser:Default;id:MD5", $cgi, {Directory => "$CONFIG::tmp_dir"} );

  my $current_dir = $session->param('&current_dir');

  my @tmp_files = ($cgi->param('one_file'));
  @tmp_files = grep {$_ ne '..'} @tmp_files;
  my @file_names = mylib::get_files($current_dir, \@tmp_files);

  @file_names = grep {-f "$current_dir/$_"} @file_names;

  my @files = ();
  map { push @files, {NAME => $_} } @file_names;

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_email_file_form.tmpl", die_on_bad_params => 0, associate => [$cgi]);
  $templ->param(
                 FILES => \@files,
	       );

  my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}

#####################################################################################################################
# EMAIL FILE
#####################################################################################################################
sub email_file
{
  my ($cgi) = @_;

  my $session = new CGI::Session("dr:File;ser:Default;id:MD5", $cgi, {Directory => "$CONFIG::tmp_dir"} );

  my $current_dir = $session->param('&current_dir');

  my @tmp_files = $cgi->param('file');
  @tmp_files = grep {$_ ne '..'} @tmp_files;
  my @file_names = mylib::get_files($current_dir, \@tmp_files);
  @file_names = grep {-f "$current_dir/$_"} @file_names;

  my @results = ();

  if (@file_names)
  {
    my $email = $cgi->param('email');
    my $subject = $cgi->param('subject');
    my $content = $cgi->param('content');

    $email =~ s/[\r\n]//gs;
    $subject =~ s/[\r\n]//gs;
    $content =~ s/\r//gs;

    my $boundary = mylib::gen_pass(20);
    my $email_tmpl = <<TMPL;
From: $CONFIG::mail_master
To: $email
Subject: $subject
Mime-Version: 1.0
Content-Type: multipart/mixed;
	boundary="$boundary"


--$boundary
Content-Type: text/plain
Content-Transfer-Encoding: 8bit

$content

--$boundary
Content-Type: application/x-zip-compressed; name="$file_names[0]"
Content-Disposition: attachment; filename="$file_names[0]"
Content-Transfer-Encoding: base64

#FILE#

--$boundary--
TMPL

    open FILE, ">$CONFIG::tmp_dir/eml\_$boundary";
    flock FILE, 2;
    print FILE $email_tmpl;
    close FILE;

    `perl admin.cgi sendmail $CONFIG::tmp_dir/eml\_$boundary $current_dir/$file_names[0] >/dev/null`;

    my @error = ();
    push @results, {NAME => "$file_names[0]", SUCCESS => (@error) ? 0 : 1, ERROR => \@error};
  }

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_email_file_results.tmpl", die_on_bad_params => 0, associate => [$cgi]);

  $templ->param(
                 RESULTS => \@results,
	       );

  my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}

#####################################################################################################################
# RENAME FILE FORM
#####################################################################################################################
sub rename_file_form
{
  my ($cgi) = @_;

  my $session = new CGI::Session("dr:File;ser:Default;id:MD5", $cgi, {Directory => "$CONFIG::tmp_dir"} );

  my $current_dir = $session->param('&current_dir');

  my @tmp_files = ();
  if ($cgi->param('one_file'))
  {
    push @tmp_files, $cgi->param('one_file');
  }
  else
  {
    @tmp_files = $cgi->param('file');
  }
  @tmp_files = grep {$_ ne '..'} @tmp_files;

  my @file_names = mylib::get_files($current_dir, \@tmp_files);

  my @files = ();
  map { push @files, {NAME => $_} } @file_names;

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_rename_file_form.tmpl", die_on_bad_params => 0, associate => [$cgi]);
  $templ->param(
                 FILES => \@files,
                 MULTIFILES => (1 < scalar @files),
	       );

  my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}

#####################################################################################################################
# RENAME FILE
#####################################################################################################################
sub rename_file
{
  my ($cgi) = @_;

  my $session = new CGI::Session("dr:File;ser:Default;id:MD5", $cgi, {Directory => "$CONFIG::tmp_dir"} );

  my $current_dir = $session->param('&current_dir');

  my @tmp_files = $cgi->param('file');
  @tmp_files = grep {$_ ne '..'} @tmp_files;

  my @file_names = mylib::get_files($current_dir, \@tmp_files);

  my $newname = $cgi->param('newname');

  my @results = ();
  if (scalar(@tmp_files) > 1)
  {
    $newname .= '/' if ($newname !~ m~[\\/]$~);
    my $newfullpath = mylib::synchronize_paths($current_dir, $newname);
    if (! -d $newfullpath)
    {
      print_error($cgi, {ERROR => "$newname - no such deirectory."});
    }

    for my $file_name (@file_names)
    {
      my @error = mylib::rename_file("$current_dir/$file_name", "$newfullpath$file_name", int $cgi->param('overwrite'));
      push @results, {OLDNAME => $file_name, NEWNAME => "$newname$file_name" , SUCCESS => (@error) ? 0 : 1, ERROR => \@error};
    }
  }
  else
  {
    my $newpath = '';
    my $newfile = '';
    if (-d $newname)
    {
      $newpath = $newname;
    }
    else
    {
      ($newpath, $newfile) = $newname =~ m~(.*[\\/])?([^\\/]+)~;
    }
    my $newfullpath = mylib::synchronize_paths($current_dir, $newpath);

    if (! -d $newfullpath)
    {
      print_error($cgi, {ERROR => "$newpath - no such deirectory."});      
    }

    my @error = mylib::rename_file("$current_dir/$file_names[0]", "$newfullpath$newfile", int $cgi->param('overwrite'));
    push @results, {OLDNAME => "$file_names[0]", NEWNAME => "$newname", SUCCESS => (@error) ? 0 : 1, ERROR => \@error};
  }

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_rename_file_results.tmpl", die_on_bad_params => 0, associate => [$cgi]);

  $templ->param(
                 RESULTS => \@results,
	       );

  my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}

#####################################################################################################################
# COPY FILE FORM
#####################################################################################################################
sub copy_file_form
{
  my ($cgi) = @_;

  my $session = new CGI::Session("dr:File;ser:Default;id:MD5", $cgi, {Directory => "$CONFIG::tmp_dir"} );

  my $current_dir = $session->param('&current_dir');

  my @tmp_files = ();
  if ($cgi->param('one_file'))
  {
    push @tmp_files, $cgi->param('one_file');
  }
  else
  {
    @tmp_files = $cgi->param('file');
  }
  @tmp_files = grep {$_ ne '..'} @tmp_files;

  my @file_names = mylib::get_files($current_dir, \@tmp_files);

  my @files = ();
  map { push @files, {NAME => $_} } @file_names;

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_copy_file_form.tmpl", die_on_bad_params => 0, associate => [$cgi]);
  $templ->param(
                 FILES => \@files,
                 MULTIFILES => (1 < scalar @files),
	       );

  my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}

#####################################################################################################################
# COPY FILE
#####################################################################################################################
sub copy_file
{
  my ($cgi) = @_;

  my $session = new CGI::Session("dr:File;ser:Default;id:MD5", $cgi, {Directory => "$CONFIG::tmp_dir"} );

  my $current_dir = $session->param('&current_dir');

  my @tmp_files = $cgi->param('file');
  @tmp_files = grep {$_ ne '..'} @tmp_files;

  my @file_names = mylib::get_files($current_dir, \@tmp_files);

  my $newname = $cgi->param('newname');

  my @results = ();
  if (scalar(@tmp_files) > 1)
  {
    $newname .= '/' if ($newname !~ m~[\\/]$~);
    my $newfullpath = mylib::synchronize_paths($current_dir, $newname);
    if (! -d $newfullpath)
    {
      print_error($cgi, {ERROR => "$newname - no such deirectory."});
    }

    for my $file_name (@file_names)
    {
      my @error = mylib::copy_file("$current_dir/$file_name", "$newfullpath$file_name", int $cgi->param('overwrite'));
      push @results, {OLDNAME => $file_name, NEWNAME => "$newname$file_name" , SUCCESS => (@error) ? 0 : 1, ERROR => \@error};
    }
  }
  else
  {
    my ($newpath, $newfile) = $newname =~ m~(.*[\\/])?([^\\/]+)~;
    my $newfullpath = mylib::synchronize_paths($current_dir, $newpath);

    if (! -d $newfullpath)
    {
      print_error($cgi, {ERROR => "$newpath - no such deirectory."});      
    }

    my @error = mylib::copy_file("$current_dir/$file_names[0]", "$newfullpath$newfile", int $cgi->param('overwrite'));
    push @results, {OLDNAME => "$file_names[0]", NEWNAME => "$newname", SUCCESS => (@error) ? 0 : 1, ERROR => \@error};
  }

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_copy_file_results.tmpl", die_on_bad_params => 0, associate => [$cgi]);

  $templ->param(
                 RESULTS => \@results,
	       );

  my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}

#####################################################################################################################
# ARCHIVE FILE FORM
#####################################################################################################################
sub archive_file_form
{
  my ($cgi, @error) = @_;

  my $session = new CGI::Session("dr:File;ser:Default;id:MD5", $cgi, {Directory => "$CONFIG::tmp_dir"} );

  my $current_dir = $session->param('&current_dir');

  my @tmp_files = ();
  if ($cgi->param('one_file'))
  {
    push @tmp_files, $cgi->param('one_file');
  }
  else
  {
    @tmp_files = $cgi->param('file');
  }
  @tmp_files = grep {$_ ne '..'} @tmp_files;

  my @file_names = mylib::get_files($current_dir, \@tmp_files);

  if (@file_names == 1 && !defined $cgi->param('filename'))
  {
    $cgi->param('filename', "$file_names[0].tar.gz");
  }

  my @files = ();
  map { push @files, {NAME => $_} } @file_names;

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_archive_file_form.tmpl", die_on_bad_params => 0, associate => [$cgi]);
  $templ->param(
                 FILES => \@files,
                 MULTIFILES => (1 < scalar @files),
                 ERROR => \@error,
	       );

  my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}

#####################################################################################################################
# ARCHIVE FILE
#####################################################################################################################
sub archive_file
{
  my ($cgi) = @_;

  my @error = ();

  my $session = new CGI::Session("dr:File;ser:Default;id:MD5", $cgi, {Directory => "$CONFIG::tmp_dir"} );

  my $current_dir = $session->param('&current_dir');

  my @tmp_files = $cgi->param('file');
  @tmp_files = grep {$_ ne '..'} @tmp_files;

  my @file_names = mylib::get_files($current_dir, \@tmp_files);

  my $newname = $cgi->param('filename');

  my @results = ();

  my ($newpath, $newfile) = $newname =~ m~(.*[\\/])?([^\\/]+)~;
  my $newfullpath = mylib::synchronize_paths($current_dir, $newpath);
  $newfile =~ s/^\s*(.*)\s*$/$1/;

  if (!$newfile)
  {
    push @error, {ERROR => 'plase enter archive filename.'}
  }

  if (! -d $newfullpath)
  {
    push @error, {ERROR => "$newpath - no such deirectory."};      
  }

  if ((! int $cgi->param('overwrite')) && (-e "$newfullpath$newfile"))
  {
    push @error, {ERROR => "$newname already exists, to overwrite it check box below."};
  }

  if ((!@error) && (-e "$newfullpath$newfile") && (! -f "$newfullpath$newfile"))
  {
    push @error, {ERROR => "could not overwrite $newname"};      
  }

  if (@error)
  {
    archive_file_form($cgi, @error);
    return;
  }

  my @error = mylib::archive_file($current_dir, "$newfullpath$newfile", ('"' . (join '" "', @file_names) . '"'));
  push @results, {NAME => "$newname", SUCCESS => (@error) ? 0 : 1, ERROR => \@error};

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_archive_file_results.tmpl", die_on_bad_params => 0, associate => [$cgi]);

  $templ->param(
                 RESULTS => \@results,
	       );

  my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}


#####################################################################################################################
# UNARCHIVE FILE FORM
#####################################################################################################################
sub unarchive_file_form
{
  my ($cgi, @error) = @_;

  my $session = new CGI::Session("dr:File;ser:Default;id:MD5", $cgi, {Directory => "$CONFIG::tmp_dir"} );

  my $current_dir = $session->param('&current_dir');

  my @tmp_files = ();
  if ($cgi->param('one_file'))
  {
    push @tmp_files, $cgi->param('one_file');
  }
  else
  {
    @tmp_files = $cgi->param('file');
  }
  @tmp_files = grep {$_ ne '..'} @tmp_files;
  my @file_names = mylib::get_files($current_dir, \@tmp_files);
  @file_names = grep {mylib::test_archive("$current_dir/$_")} @file_names;

  my @files = ();
  map { push @files, {NAME => $_} } @file_names;

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_unarchive_file_form.tmpl", die_on_bad_params => 0, associate => [$cgi]);
  $templ->param(
                 FILES => \@files,
                 ERROR => \@error,
	       );

  my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}

#####################################################################################################################
# UNARCHIVE FILE
#####################################################################################################################
sub unarchive_file
{
  my ($cgi) = @_;

  my @error = ();

  my $session = new CGI::Session("dr:File;ser:Default;id:MD5", $cgi, {Directory => "$CONFIG::tmp_dir"} );

  my $current_dir = $session->param('&current_dir');

  my @tmp_files = $cgi->param('file');
  @tmp_files = grep {$_ ne '..'} @tmp_files;
  my @file_names = mylib::get_files($current_dir, \@tmp_files);
  @file_names = grep {mylib::test_archive("$current_dir/$_")} @file_names;

  my @results = ();

  my @error = mylib::unarchive_file($current_dir, $file_names[0], int $cgi->param('overwrite'));
  push @results, {NAME => $file_names[0], SUCCESS => (@error) ? 0 : 1, ERROR => \@error};

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_unarchive_file_results.tmpl", die_on_bad_params => 0, associate => [$cgi]);

  $templ->param(
                 RESULTS => \@results,
	       );

  my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}

#####################################################################################################################
# DELETE FILE
#####################################################################################################################
sub delete_file
{
  my ($cgi) = @_;

  my $session = new CGI::Session("dr:File;ser:Default;id:MD5", $cgi, {Directory => "$CONFIG::tmp_dir"} );

  my $current_dir = $session->param('&current_dir');

  my @tmp_files = ();
  if ($cgi->param('one_file'))
  {
    push @tmp_files, $cgi->param('one_file');
  }
  else
  {
    @tmp_files = $cgi->param('file');
  }
  @tmp_files = grep {$_ ne '..'} @tmp_files;

  my @file_names = mylib::get_files($current_dir, \@tmp_files);

  my @results = ();

  for my $file_name (@file_names)
  {
    my @error = mylib::delete_file("$current_dir/$file_name", 0);
    push @results, {NAME => $file_name, SUCCESS => (@error) ? 0 : 1, ERROR => \@error};
  }

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_delete_file_results.tmpl", die_on_bad_params => 0, associate => [$cgi]);

  $templ->param(
                 RESULTS => \@results,
	       );

  my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}

#####################################################################################################################
# ERASE FILE
#####################################################################################################################
sub erase_file
{
  my ($cgi) = @_;

  my @tmp_files = ();
  if ($cgi->param('one_file'))
  {
    push @tmp_files, $cgi->param('one_file');
  }
  else
  {
    @tmp_files = $cgi->param('file');
  }
  @tmp_files = grep {$_ ne '..'} @tmp_files;

  my @file_names = mylib::get_files($CONFIG::recycle_dir, \@tmp_files);

  my @results = ();

  for my $file_name (@file_names)
  {
    my @error = mylib::delete_file("$CONFIG::recycle_dir/$file_name", 1);
    push @results, {NAME => $file_name, SUCCESS => (@error) ? 0 : 1, ERROR => \@error};
  }

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_erase_file_results.tmpl", die_on_bad_params => 0, associate => [$cgi]);

  $templ->param(
                 RESULTS => \@results,
	       );

  my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}

#####################################################################################################################
# RESTORE FILE
#####################################################################################################################
sub restore_file
{
  my ($cgi) = @_;

  my @tmp_files = ();
  if ($cgi->param('one_file') ne '')
  {
    push @tmp_files, ($cgi->param('one_file'));
  }
  else
  {
    @tmp_files = $cgi->param('file');
  }

  @tmp_files = grep {$_ ne '..'} @tmp_files;

  my @file_names = mylib::get_files($CONFIG::recycle_dir, \@tmp_files);


  my %data = load("$CONFIG::recycle_dir/init.dat");

  @file_names = grep {defined $data{$_}} @file_names;

  my @results = ();
  for my $file_name (@file_names)
  {
    my $restore_path = $data{$file_name}->{SOURCE_FILE};
    my @error = mylib::rename_file("$CONFIG::recycle_dir/$file_name", $restore_path, 0);
    $restore_path =~ s/^$ROOT_DIR//s;
    push @results, {OLDNAME => $file_name, NEWNAME => $restore_path, SUCCESS => (@error) ? 0 : 1, ERROR => \@error};
  }

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_restore_file_results.tmpl", die_on_bad_params => 0, associate => [$cgi]);

  $templ->param(
                 RESULTS => \@results,
	       );

  my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}

#####################################################################################################################
# CREATE FILE FORM
#####################################################################################################################
sub create_file_form
{
  my ($cgi, @error) = @_;

  my $session = new CGI::Session("dr:File;ser:Default;id:MD5", $cgi, {Directory => "$CONFIG::tmp_dir"} );

  my $current_dir = $session->param('&current_dir');

  mylib::prepare_cgi_view($cgi);
  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_create_file_form.tmpl", die_on_bad_params => 0, associate => [$cgi]);
  $templ->param(
                 ERROR => \@error,
	       );

  my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}

#####################################################################################################################
# CREATE FILE
#####################################################################################################################
sub create_file
{
  my ($cgi) = @_;

  my @error = ();

  my $session = new CGI::Session("dr:File;ser:Default;id:MD5", $cgi, {Directory => "$CONFIG::tmp_dir"} );

  my $current_dir = $session->param('&current_dir');

  my $newname = $cgi->param('filename');

  my ($newpath, $newfile) = $newname =~ m~(.*[\\/])?([^\\/]+)~;

  my $newfullpath = mylib::synchronize_paths($current_dir, $newpath);

  if (!$newfile) { push @error, {ERROR => 'Please enter new file name.'} }

  if (! -d $newfullpath) { push @error, {ERROR => "$newpath: no such directory."} }

  if ((int($cgi->param('overwrite'))) && (-e "$newfullpath$newfile") && (! -f "$newfullpath$newfile")) 
  { push @error, {ERROR => "could not overvrite object $newname" } }

  if ((!@error) && (! int($cgi->param('overwrite'))) && (-e "$newfullpath$newfile") )
  {
    push @error, {ERROR => "$newname already exists, to overwrite $newname check box below."}
  }

  if (@error)
  {
    create_file_form($cgi, @error);
    return;
  }

  my @results = ();

  my @error = mylib::write_file("$newfullpath$newfile", $cgi->param('content'));
  push @results, { NAME => "$newname", SUCCESS => (@error) ? 0 : 1, ERROR => \@error};

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_create_file_results.tmpl", die_on_bad_params => 0, associate => [$cgi]);

  $templ->param(
                 RESULTS => \@results,
	       );

  my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}


#####################################################################################################################
# EDIT FILE FORM
#####################################################################################################################
sub edit_file_form
{
  my ($cgi, @error) = @_;

  my $session = new CGI::Session("dr:File;ser:Default;id:MD5", $cgi, {Directory => "$CONFIG::tmp_dir"} );

  my $current_dir = $session->param('&current_dir');

  my $newname = $cgi->param('filename');
  $newname = (split /[\\\/]/, $newname)[-1];

  if (! -f "$current_dir/$newname")
  {
    print_error($cgi, {ERROR => "$newname: no such file."});
    return;
  }

  if (! -T "$current_dir/$newname")
  {
    print_error($cgi, {ERROR => "$newname file is not text file."});
    return;
  }

  open FILE, "<$current_dir/$newname" or print_error($cgi, {ERROR => "$!"});;
  flock FILE, 2;
  my $content = join '', <FILE>;
  close FILE;

  $cgi->param('filename', $newname) if (!defined $cgi->param('filename'));
  $cgi->param('content', $content) if (!defined $cgi->param('content'));
  $cgi->param('overwrite', 1) if (!defined $cgi->param('overwrite'));


  mylib::prepare_cgi_view($cgi);
  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_edit_file_form.tmpl", die_on_bad_params => 0, associate => [$cgi]);
  $templ->param(
                 ERROR => \@error,
	       );

  my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}

#####################################################################################################################
# EDIT FILE
#####################################################################################################################
sub edit_file
{
  my ($cgi) = @_;

  my @error = ();

  my $session = new CGI::Session("dr:File;ser:Default;id:MD5", $cgi, {Directory => "$CONFIG::tmp_dir"} );

  my $current_dir = $session->param('&current_dir');

  my $newname = $cgi->param('filename');

  my ($newpath, $newfile) = $newname =~ m~(.*[\\/])?([^\\/]+)~;

  my $newfullpath = mylib::synchronize_paths($current_dir, $newpath);

  if (!$newfile) { push @error, {ERROR => 'Please enter new file name.'} }

  if (! -d $newfullpath) { push @error, {ERROR => "$newpath: no such directory."} }

  if ((int($cgi->param('overwrite'))) && (-e "$newfullpath$newfile") && (! -f "$newfullpath$newfile")) 
  { push @error, {ERROR => "could not overvrite object $newname" } }

  if ((!@error) && (! int($cgi->param('overwrite'))) && (-e "$newfullpath$newfile") )
  {
    push @error, {ERROR => "$newname already exists, to overwrite $newname check box below."}
  }

  if (@error)
  {
    create_file_form($cgi, @error);
    return;
  }

  my @results = ();

  my @error = mylib::write_file("$newfullpath$newfile", $cgi->param('content'));
  push @results, { NAME => "$newname", SUCCESS => (@error) ? 0 : 1, ERROR => \@error};

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_edit_file_results.tmpl", die_on_bad_params => 0, associate => [$cgi]);

  $templ->param(
                 RESULTS => \@results,
	       );

  my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}

#####################################################################################################################
# SEARCH FILE FORM
#####################################################################################################################
sub search_file_form
{
  my ($cgi, @error) = @_;

  my $session = new CGI::Session("dr:File;ser:Default;id:MD5", $cgi, {Directory => "$CONFIG::tmp_dir"} );

  my $current_dir = $session->param('&current_dir');

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_search_file_form.tmpl", die_on_bad_params => 0, associate => [$cgi]);
  $templ->param(
                 ERROR => \@error,
	       );

  my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}


#####################################################################################################################
# SEARCH FILE
#####################################################################################################################
sub search_file
{
  my ($cgi) = @_;

  my @error = ();

  my $session = new CGI::Session("dr:File;ser:Default;id:MD5", $cgi, {Directory => "$CONFIG::tmp_dir"} );

  my $current_dir = $session->param('&current_dir');

  if ($cgi->param('wait'))
  {
    if (kill( 0 => $cgi->param('wait') ))
    {
      my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_wait_results.tmpl", die_on_bad_params => 0, associate => [$cgi]);
      $templ->param(
                     MESSAGE => "SEARCHING...",
                     ACTION => $cgi->param('act'),
                     WAIT => $cgi->param('wait'),
    	           );
      my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
      print "Content-type: text/html\n\n", $template->output();
    }
    else
    {


      my @results = ();
      open FILE, "<$CONFIG::tmp_dir/search_" . $cgi->param('wait');
      flock FILE, 2;
      while (my $file = <FILE>)
      {
        chomp $file;
        next if ($file =~ /^error:/);
        my ($found_dir, $found_file) = $file =~ m~(.*[\\/])([^\\/]*)~;
#        $file =~ s/^$current_dir\///;
#        my $search_from = $ROOT_DIR;
#        $file =~ s/^$search_from//;
#        $found_dir =~ s/^$search_from//;

        push @results, {DIR => $found_dir, FILENAME => $found_file, PATH => $file};
      }
      close FILE;

      my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_search_file_results.tmpl", die_on_bad_params => 0, associate => [$cgi]);
      $templ->param(
                     RESULTS => \@results,
    	           );
      my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
      print "Content-type: text/html\n\n", $template->output();
#      unlink("$CONFIG::tmp_dir/search_" . $cgi->param('wait'));
    }
  }
  else
  {
    my $search_from = $current_dir;
    if (int $cgi->param('begin'))
    {
      $search_from = $ROOT_DIR;
    }

    my $search_file = $cgi->param('name');
    my $search_text = $cgi->param('text');

    if ($search_file eq '' && $search_text eq '')
    {
      push @error, {ERROR => 'Please enter search file pattern'};
    }

    if (@error)
    {
      search_file_form($cgi, @error);
      return;
    }

    `perl admin.cgi searchfile "$search_from" "$search_file" "$search_text" >$CONFIG::tmp_dir/search_$$\_start`;

    open FILE, "$CONFIG::tmp_dir/search_$$\_start";
    flock FILE, 2;
    my $wait_pid = <FILE>;
    close FILE;
    unlink("$CONFIG::tmp_dir/search_$$\_start");

    my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_wait_results.tmpl", die_on_bad_params => 0, associate => [$cgi]);
    $templ->param(
                   MESSAGE => "SEARCHING...",
                   ACTION => $cgi->param('act'),
                   WAIT => $wait_pid,
  	         );
    my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
    print "Content-type: text/html\n\n", $template->output();
  }
}

#####################################################################################################################
# FOLDERS TREE
#####################################################################################################################
sub tree
{
  my ($cgi, @error) = @_;

  use vars qw{$current_dir};
  my $session = new CGI::Session("dr:File;ser:Default;id:MD5", $cgi, {Directory => "$CONFIG::tmp_dir"} );

  $current_dir = $session->param('&current_dir');

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_tree.tmpl", die_on_bad_params => 0, associate => [$cgi]);
  $templ->param(
                 TREE => make_tree("$ROOT_DIR/"),
	       );

  my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}


#####################################################################################################################
# CREATE FOLDER FORM
#####################################################################################################################
sub create_folder_form
{
  my ($cgi, @error) = @_;

  my $session = new CGI::Session("dr:File;ser:Default;id:MD5", $cgi, {Directory => "$CONFIG::tmp_dir"} );

  my $current_dir = $session->param('&current_dir');

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_create_folder_form.tmpl", die_on_bad_params => 0, associate => [$cgi]);
  $templ->param(
                 ERROR => \@error,
	       );

  my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}

#####################################################################################################################
# CREAT FOLDER
#####################################################################################################################
sub create_folder
{
  my ($cgi) = @_;

  my @error = ();

  my $session = new CGI::Session("dr:File;ser:Default;id:MD5", $cgi, {Directory => "$CONFIG::tmp_dir"} );

  my $current_dir = $session->param('&current_dir');

  my $newname = $cgi->param('filename');

  my ($newpath, $newfile) = $newname =~ m~(.*[\\/])?([^\\/]+)~;

  $newfile =~ s/^\s*(.*)\s*$/$1/;

  my $newfullpath = mylib::synchronize_paths($current_dir, $newpath);

  if (!$newfile) { push @error, {ERROR => 'Please enter new folder name.'} }

  if (! -d $newfullpath) { push @error, {ERROR => "$newpath: no such directory."} }

  if ((!@error) && (-e "$newfullpath$newfile"))
  {
    push @error, {ERROR => "$newname already exists"}
  }

  if (@error)
  {
    create_folder_form($cgi, @error);
    return;
  }

  my @results = ();

  my @error = mylib::create_folder("$newfullpath$newfile");
  push @results, { NAME => "$newname", SUCCESS => (@error) ? 0 : 1, ERROR => \@error};

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_create_folder_results.tmpl", die_on_bad_params => 0, associate => [$cgi]);

  $templ->param(
                 RESULTS => \@results,
	       );

  my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}


#####################################################################################################################
# UNIX shell
#####################################################################################################################
sub unix_shell
{
  my ($cgi, @error) = @_;

  my $session = new CGI::Session("dr:File;ser:Default;id:MD5", $cgi, {Directory => "$CONFIG::tmp_dir"} );

  my $current_dir = $session->param('&current_dir');

  chdir($current_dir);
  my $cmd = $cgi->param('cmd');

  $cmd =~ s/\$/\\\$/gs;
  $cmd =~ s/\@/\\\@/gs;
  $cmd =~ s/\%/\\\%/gs;

  my $result = `$cmd 2>&1`;

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_unix_shell.tmpl", die_on_bad_params => 0, associate => [$cgi]);
  $templ->param(
                 RESULT => $result,
	       );

  my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}

#####################################################################################################################
# CHECK LOGIN
#####################################################################################################################
sub check_login
{
  my ($cgi) = @_;

  my $login = $cgi->cookie('admin_login') || $cgi->param('login');
  my $try_password = $cgi->cookie('admin_password') || crypt($cgi->param('password'), $cgi->param('login'));

  my $password = $CONFIG::admin_password;
  my $login = $CONFIG::admin_login;

  if($try_password ne crypt($password, $login) || !$try_password || !$password)
  {
    my @error = ();
    if ($cgi->param('logining'))
    {
      push @error, {ERROR => 'Invalid Login or Password'};
      mylib::log_action("Login Fail with Login: $login Password: " . $cgi->param('password'));
    }
    login_form($cgi, @error);
    exit;
  }

  if ($cgi->param('login'))
  {
    mylib::log_action("Login Success");
  }

  my $cookie = $cgi->cookie(-name => 'admin_password', -value => $try_password);
  print "Set-Cookie: $cookie\n";
  my $cookie = $cgi->cookie(-name => 'admin_login', -value => $login);
  print "Set-Cookie: $cookie\n";
}

#####################################################################################################################
# LOGIN FORM
#####################################################################################################################
sub login_form
{
  my ($cgi, @error) = @_;

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_login_form.tmpl", die_on_bad_params => 0, associate => [$cgi]);

  my $login = $cgi->cookie('login');
  $templ->param(
                 ERROR => \@error,
                 LOGIN => $login,
	       );
  my $template = mylib::adm_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
}

#####################################################################################################################
# LOGOUT
#####################################################################################################################
sub logout
{
  my ($cgi) = @_;

  my $cookie1 = $cgi->cookie(-name => "admin_login",
                        -value => "",
                        -expires => '-1d'
                        );
  my $cookie2 = $cgi->cookie(-name => 'admin_password',
                        -value => "",
                        -expires => '-1d'
                        );

  my $cookie = $cookie1."\nSet-Cookie: $cookie2";
  print $cgi->redirect(-cookie => $cookie,
                       -uri    => "http://$ENV{HTTP_HOST}$ENV{SCRIPT_NAME}"
                      );

  return;
}

#####################################################################################################################
# PRINT ERROR
#####################################################################################################################
sub print_error
{
  my ($cgi, @error) = @_;

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_error.tmpl", die_on_bad_params => 0, associate => [$cgi]);

  $templ->param(
                 ERROR => \@error,
      	       );

  my $template = mylib::adm_popup_main_tmpl($cgi, $templ, 0);
  print "Content-type: text/html\n\n", $template->output();
  exit;
}

sub log_error
{
  my ($error, $line) = @_;

  open FILE, ">>./error_log";
  print FILE "\nError: $error\nFile:" . __FILE__ . "\nLine: $line";
  close FILE;
}

sub close_window_and_reload_parent
{
  return qq{Content-type: text/html

<html>
<script>
opener.location.reload();
window.close();
</script>
</html>
};
}


#####################################################################################################################
# BACKGROUND SUBS
#####################################################################################################################
sub sendmail
{
  my ($template_file, $send_file) = @ARGV;

  open FILE, "<$template_file";
  flock FILE, 2;
  my $template = join '', <FILE>;
  close FILE;

  open FILE, "<$send_file";
  flock FILE, 2;
  binmode FILE;
  my $file = join '', <FILE>;
  close FILE;

  my $base64_file = encode_base64($file);

  $template =~ s/#FILE#/$base64_file/;

  open MAIL, "|$CONFIG::sendmail -t -oi";
  print MAIL $template;
  close MAIL;

  unlink $template_file;
}


sub searchfile
{
  use vars qw{$file_pattern $text_pattern}; 
  (my $from_dir, $file_pattern, $text_pattern) = @ARGV;

  $file_pattern =~ s/([^a-z0-9])/\\$1/gsi;
  $text_pattern =~ s/([^a-z0-9])/\\$1/gsi;

  if ($from_dir !~ /[\\\/]$/)
  {
    $from_dir .= '/';
  }

  open FH, ">$CONFIG::tmp_dir/search_$$";
  flock FH, 2;
  print FH '';
  close FH;

  dir($from_dir);
}


sub dir
{
  my ($dir) = @_;

  opendir DIR, $dir or print_found_log("error: $dir $!\n");
  my @files = grep {!/^\./} readdir DIR;
  closedir DIR;

  for my $file (@files)
  {
    if ($file_pattern && $file =~ /$file_pattern/si)
    {
      print_found_log("$dir$file\n");
    }

    if (-d "$dir$file")
    {
      dir("$dir$file/");
    }
    elsif (-f "$dir$file")
    {
      if ($text_pattern)
      {
        open FILE, "$dir$file" or print_found_log("error: $dir$file $!\n");
        my $txt = join '', <FILE>;
        close FILE;
        if ($txt =~ /$text_pattern/si)
        {
          print_found_log("$dir$file\n");
        }
      }
    }
  }
}


sub make_tree
{
  my ($dir, $level) = @_;

  opendir DIR, $dir;
  my @files = grep {!/^\./} readdir DIR;
  closedir DIR;

  my @dirs = ();

  for my $file (sort @files)
  {
    if (-d "$dir$file")
    {
      my $path = "$dir$file";
      $path =~ s/^$ROOT_DIR//gs;
      my @safe_file = ($file, $path);
      mylib::prepare_view(\@safe_file);
      push @dirs, { NAME => $safe_file[0], PATH => $safe_file[1], DIR => "$dir$file", image_dir => $CONFIG::images_dir, CURRENT => ($current_dir eq "$dir$file")};
    }
  }

  for (my $i = 0; $i < @dirs; $i++)
  {
    push @{$level}, { LAST => ($i == @dirs - 1), image_dir => $CONFIG::images_dir };
    $dirs[$i]->{LEVELS} = \@{$level};
    $dirs[$i]->{LAST} = ($i == @dirs - 1);
    $dirs[$i]->{SUBS} = make_tree("$dirs[$i]->{DIR}/", $level);
    pop @{$level};
  }

  my $templ = HTML::Template->new(filename => "$CONFIG::tmpl_path/adm_tree_leaf.tmpl", die_on_bad_params => 0);

  $templ->param(
                 FOLDERS => \@dirs,
	       );

  return $templ->output();
}

sub print_found_log
{
  open FH, ">>$CONFIG::tmp_dir/search_$$";
  flock FH, 2;
  print FH @_;
  close FH;
}