#!/usr/bin/perl
###############################################################################
# Programming by HotCGIScripts Team | http://www.hotcgiscripts.com/           #
# All Right Reserved                                                          #
###############################################################################

use strict;
use CGI;
use CGI::Carp qw{fatalsToBrowser};
use Time::Local;

use lib './lib';
use MIME::Base64;
use CGI::Session;
use HTML::Template;
use DataStore qw{load save};
use mylib;
use CONFIG;

main();

sub main
{
  my $cgi = new CGI;

  my $act = $cgi->param('act');

  if($act eq 'home')
  {
    &home($cgi);
  }

####################################
# SETTINGS MANAGEMENT
####################################
  elsif ($act eq 'edit_settings_form')
  {
    &edit_settings_form($cgi);
  }
  elsif ($act eq 'edit_settings')
  {
    &edit_settings($cgi);
  }
  else
  {
    &home($cgi);
  }
}

#####################################################################################################################
# HOME
#####################################################################################################################
sub home
{
  my ($cgi) = @_;

  edit_settings_form($cgi);
}

#####################################################################################################################
# EDIT SETTINGS FORM
#####################################################################################################################
sub edit_settings_form
{
  my ($cgi, @error) = @_;

  my $install_dir = $ENV{SCRIPT_FILENAME};
  $install_dir =~ s/install\.cgi$//;
  my $images_dir = $install_dir;
  $images_dir =~ s/$ENV{DOCUMENT_ROOT}//;

  my $templ = HTML::Template->new(filename => "./templates/setup/settings_form.tmpl", die_on_bad_params => 0, associate => [$cgi]);
  $templ->param(
                 admin_login => $cgi->param('admin_login') || $CONFIG::admin_login,
                 admin_password => $cgi->param('admin_password') || $CONFIG::admin_password,
		 admin_cpassword => $cgi->param('admin_cpassword') || $CONFIG::admin_password,
                 admin_mail => $cgi->param('admin_mail') || $CONFIG::admin_mail,
                 mail_master => $cgi->param('mail_master') || $CONFIG::mail_master,
                 install_dir => $cgi->param('install_dir') || $CONFIG::base_dir || $install_dir,
                 images_url => $cgi->param('images_url') || $CONFIG::images_dir || "http://$ENV{HTTP_HOST}".$images_dir."i/",
		 sendmail => $cgi->param('sendmail') || $CONFIG::sendmail,
		 tar => $cgi->param('tar') || $CONFIG::tar,
                 ERROR => \@error,
	       );

  print "Content-type: text/html\n\n", $templ->output();
}

#####################################################################################################################
# EDIT SETTINGS
#####################################################################################################################
sub edit_settings
{
  my ($cgi) = @_;
  my @error = ();

  if ($cgi->param('admin_login') eq '') { push @error, {ERROR => "Please enter Administrator Login"}};
  if ($cgi->param('admin_password') eq '') { push @error, {ERROR => "Please enter Administrator Password"}}
  elsif ($cgi->param('admin_password') ne $cgi->param('admin_cpassword')) { push @error, {ERROR => "Confirm Administrator Password Correctly"}};

  my $install_dir = $cgi->param('install_dir');
  $install_dir =~ s/\/$//;
  if (! -d $install_dir) { push @error, {ERROR => "Specified Installetinon folder not exists."}}
  else
  {
    if (! -e "$install_dir/admin.cgi") { push @error, {ERROR => "Could not find $install_dir/admin.cgi"}};
    if (! -e "$install_dir/lib") { push @error, {ERROR => "Could not find $install_dir/lib"}};
    if (! -e "$install_dir/templates") { push @error, {ERROR => "Could not find $install_dir/templates"}};
    if (! -e "$install_dir/tmp") { push @error, {ERROR => "Could not find $install_dir/tmp"}};
    if (! -e "$install_dir/RECYCLE") { push @error, {ERROR => "Could not find $install_dir/RECYCLE"}};
  }
  my $images_url = $cgi->param('images_url');
  my ($images_dir) = (mylib::parse_url($images_url))[5];
  $images_url .= '/' if ($images_url !~ m~/$~);

  if (! -d "$ENV{DOCUMENT_ROOT}$images_dir") { push @error, {ERROR => "Specified Images URL $images_url relating $ENV{DOCUMENT_ROOT}$images_dir not exists."}}
  my $tar = $cgi->param('tar');
  my $result = `$tar 2>&1`;

  if (!$result || $result =~ /Command not found/si)
  {
    push @error, {ERROR => "TAR is unavailable by path specified."}
  }

  my $sendmail = $cgi->param('sendmail');

  if (@error)
  {
    edit_settings_form($cgi, @error);
    return;
  }

  open FILE, "<./lib/CONFIG.pm";
  flock FILE, 2;
  my $txt = join '', <FILE>;
  close FILE;

  my $admin_login = $cgi->param('admin_login');
  $admin_login =~ s/([^0-9a-z])/\\$1/gsi;
  $txt =~ s/\$admin_login\s*=\s*".*?";/\$admin_login = "$admin_login";/s;

  my $admin_password = $cgi->param('admin_password');
  $admin_password =~ s/([^0-9a-z])/\\$1/gsi;
  $txt =~ s/\$admin_password\s*=\s*".*?";/\$admin_password = "$admin_password";/s;

  my $admin_mail = $cgi->param('admin_mail');
  $admin_mail =~ s/([^0-9a-z])/\\$1/gsi;
  $txt =~ s/\$admin_mail\s*=\s*".*?";/\$admin_mail = "$admin_mail";/s;

  my $mail_master = $cgi->param('mail_master');
  $mail_master =~ s/([^0-9a-z])/\\$1/gsi;
  $txt =~ s/\$mail_master\s*=\s*".*?";/\$mail_master = "$mail_master";/s;

  $install_dir =~ s/([^0-9a-z])/\\$1/gsi;
  $txt =~ s/\$base_dir\s*=\s*".*?";/\$base_dir = "$install_dir";/s;

  $images_url =~ s/([^0-9a-z])/\\$1/gsi;
  $txt =~ s/\$images_dir\s*=\s*".*?";/\$images_dir = "$images_url";/s;

  $tar =~ s/([^0-9a-z])/\\$1/gsi;
  $txt =~ s/\$tar\s*=\s*".*?";/\$tar = "$tar";/s;

  $sendmail =~ s/([^0-9a-z])/\\$1/gsi;
  $txt =~ s/\$sendmail\s*=\s*".*?";/\$sendmail = "$sendmail";/s;

  my $crypt_method = 1;
  if ($ENV{SERVER_SOFTWARE} !~ /apache/i)
  {
    $crypt_method = 2;
  }
  $txt =~ s/\$crypt_method\s*=\s*".*?";/\$crypt_method = "$crypt_method";/s;

  open FILE, ">./lib/CONFIG.pm";
  flock FILE, 1;
  print FILE $txt;
  close FILE;

  my $login_path = "http://$ENV{HTTP_HOST}$ENV{SCRIPT_NAME}";

  $login_path =~ s~install\.cgi$~admin.cgi~gs;

  my $templ = HTML::Template->new(filename => "./templates/setup/settings_done.tmpl", die_on_bad_params => 0, associate => [$cgi]);
  $templ->param(
                 LOGIN_PATH => $login_path,
               );
  print "Content-type: text/html\n\n", $templ->output();
}
