package Dir::Copy;

use strict;
use File::Copy;
require Exporter;
require DynaLoader;

use vars qw(@ISA @EXPORT $VERSION $OLD_CODE);

$VERSION = '1.0';

@ISA = qw(Exporter DynaLoader);
@EXPORT = qw(copy);

sub new 
{
  my $pkg = shift;
  my $self; { my %hash; $self = bless(\%hash, $pkg); }

  for (my $x = 0; $x <= $#_; $x += 2) 
  {
    defined($_[($x + 1)]) or croak("Dir::Copy->new() called with odd number of option parameters - should be of the form option => value");
    $self->{lc($_[$x])} = $_[($x + 1)];
  }

  $self->{attributes} = '755' if (!defined $self->{attributes});
  $self->{attributes} = oct($self->{attributes});
  $self->{overwrite} = 0 if (!defined $self->{overwrite});
  $self->{verbose} = 0 if (!defined $self->{verbose});

  return $self;
}

sub copy
{
  my ($self, $from_dir, $to_dir) = @_;

  umask(0000);

  $self->{errors} = ();

  $from_dir =~ s~[\\/]+$~~s;
  $to_dir =~ s~[\\/]+$~~s;

  if ($from_dir eq $to_dir)
  {
    push @{$self->{errors}}, "$from_dir : Can't copy to itself";
    return;
  }

  if (-f $from_dir)
  {
    my $from_file = $from_dir;
    my $to_file = $to_dir;
    if (-d $to_dir)
    {
      my $new_file_name = (split m~[\\/]~, $from_file)[-1];
      $to_file = "$to_dir/$new_file_name";
    }

    if ($from_dir eq $to_dir)
    {
      push @{$self->{errors}}, "$from_dir : Can't copy to itself";
      return;
    }

    if ((-f $to_file) && (!$self->{overwrite}))
    {
      push @{$self->{errors}}, "$to_file : Not Overwriten";
    }
    else
    {
      print "copy $from_file to $to_file" if $self->{verbose};
      my $mode = (stat("$from_file"))[2];
      File::Copy::copy("$from_file", "$to_file") or push @{$self->{errors}}, "Can't copy $from_file to $to_file : $!";
      chmod($mode, "$to_file");
    }
    return;
  }

  my $mode = (stat("$from_dir"))[2];

  my $err_flag = 0;
  if (! -e $to_dir)
  {
    mkdir($to_dir, $mode) or $err_flag = 1;
    if ($err_flag)
    {
      push @{$self->{errors}}, "Can't create directory $to_dir : $!";
      return;
    }
  }
  elsif (! -d $to_dir)
  {
    push @{$self->{errors}}, "Not directory: $to_dir";
    return;
  }
  else
  {
    my $new_dir_name = $from_dir;
    $new_dir_name = (split m~[\\/]~, $new_dir_name)[-1];
    if (! -e "$to_dir/$new_dir_name")
    {
      mkdir("$to_dir/$new_dir_name", $mode) or $err_flag = 1;
      if ($err_flag)
      {
        push @{$self->{errors}}, "Can't create directory $to_dir/$new_dir_name : $!";
        return;
      }
    }
    $to_dir = "$to_dir/$new_dir_name";
  }

  _dir_copy($self, $from_dir, $to_dir);
}

sub _dir_copy
{
  my ($self, $from_dir, $to_dir) = @_;

  $from_dir =~ s~[\\/]+$~~s;
  $to_dir =~ s~[\\/]+$~~s;

  my $mode = (stat("$from_dir"))[2];

  my $err_flag = 0;
  if (! -e $to_dir)
  {
    mkdir($to_dir, $mode) or $err_flag = 1;
    if ($err_flag)
    {
      push @{$self->{errors}}, "Can't create directory $to_dir : $!";
      return;
    }
  }
  elsif (! -d $to_dir)
  {
    push @{$self->{errors}}, "Not directory: $to_dir";
    return;
  }

  my $err_flag = 0;
  opendir DIR, ($from_dir) or $err_flag = 1;
  if ($err_flag)
  {
    push @{$self->{errors}}, "Can't open directory: $from_dir";
    return;
  }
  my @files = grep {$_ ne '.' && $_ ne '..'} readdir DIR;
  close DIR;

  for my $file (@files)
  {
    if (-d "$from_dir/$file")
    {
      _dir_copy($self, "$from_dir/$file", "$to_dir/$file");
    }
    else
    {
      if ((-f "$to_dir/$file") && (!$self->{overwrite}))
      {
        push @{$self->{errors}}, "$to_dir/$file : Not Overwriten";
      }
      else
      {
        print "copy $from_dir/$file to $to_dir/$file" if $self->{verbose};
        my $mode = (stat("$from_dir/$file"))[2];
        File::Copy::copy("$from_dir/$file", "$to_dir/$file") or push @{$self->{errors}}, "Can't copy $from_dir/$file to $to_dir/$file : $!";
        chmod($mode, "$to_dir/$file");
      }
    }
  }
}